/* eslint-disable camelcase */
import axios from 'axios'
import axiosRetry from 'axios-retry'
import { EventLog } from './event-log'
import ErrorHandler from './error-handler'

let $axios = null
let device_id = null
let loggingEnabled = false
let eventLog = null
let showErrorsOnScreen = false

export default {
    async install(Vue, options) {
        try {
            $axios = await initAxios(options.url, options.client_id, options.client_secret, options.retries)
        } catch (e) {
            if (showErrorsOnScreen) {
                console.log(e.message)
                ErrorHandler.screen(e.message)
            }
            if (options.error) {
                options.error(e, options)
            }
            return
        }

        device_id = options.device_id
        loggingEnabled = options.logging
        showErrorsOnScreen = options.showErrorsOnScreen
        eventLog = new EventLog($axios, device_id)

        if (loggingEnabled) {
            try {
                await eventLog.getLogTypes()
            } catch (e) {
                if (showErrorsOnScreen) {
                    ErrorHandler.screen(e.message)
                }
            }
        }

        Vue.prototype.$axios = $axios
        Vue.prototype.$twoavy = {
            log: (...args) => eventLog.log(...args).catch(e => showErrorsOnScreen ? ErrorHandler.screen(e.message) : Promise.reject(e)),
            error: (message, permanent) => ErrorHandler.screen(message, permanent)
        }

        if (options.ready) {
            options.ready()
        }
    }
}

async function initAxios(url, client_id = false, client_secret = false, retries = 3) {
    let axs = axios.create()
    axs = addInterceptors(axs, retries)

    let tokenRes = {
        data: {
            access_token: ''
        }
    }
    if (client_id && client_secret) {
        tokenRes = await axs.post(
            url + '/oauth/token',
            {
                grant_type: 'client_credentials',
                scope: '*',
                client_id,
                client_secret
            })
    }
    let instance = axios.create({
        baseURL: url + '/api',
        timeout: 5000,
        headers: {
            Authorization: 'Bearer ' + tokenRes.data.access_token,
            'Content-Type': 'application/json'
        }
    })
    return addInterceptors(instance)
}

function addInterceptors(axs, retries) {
    axiosRetry(axs, {
        retries,
        retryDelay: count => count * 500,
        retryCondition: () => true
    })
    axs.interceptors.response.use((response) => {
        return response
    }, error => {
        if (error.response) {
            const msg = error.response.data.error || error.response.data.message
            console.log(msg)
            return Promise.reject(new Error(msg))
        } else if (error.request) {
            // The request was made but no response was received
            return Promise.reject(new Error(`${error.stack}: no response received`))
        } else {
            // Something happened in setting up the request that triggered an Error
            return Promise.reject(error)
        }
    })
    return axs
}
